-----------------------------------------------------------------------------------------------
-- Client Lua Script for RedAlert
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "MatchingGame"
require "Apollo"
require "ChatSystemLib"
require "Unit"
require "Spell"
require "GroupLib"
 
-----------------------------------------------------------------------------------------------
-- RedAlert Module Definition
-----------------------------------------------------------------------------------------------
local RedAlert = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function RedAlert:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.tUnits = {}
	o.tCasts = {
		"Activating Control Point",
		"Picking Up Moodie Mask",
		"Detaching Moodie Mask",
		"Initiating Capture",
		"Activating Control Point",
	}
	o.tConfig = {}
	o.tConfig.IsVisible = true
	o.tConfig.Position = -- Default position
 	{
  		Bottom = 61,
  		Left = 6,
  		Right = 248,
  		Top = 4,
 	}

    return o
end

function RedAlert:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- RedAlert OnLoad
-----------------------------------------------------------------------------------------------
function RedAlert:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("RedAlert.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- RedAlert OnDocLoaded
-----------------------------------------------------------------------------------------------
function RedAlert:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "RedAlertForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		else
			if self.tConfig ~= nil then
				if self.tConfig.Position ~= nil then
					self.wndMain:SetAnchorOffsets(
					self.tConfig.Position.Left,
					self.tConfig.Position.Top,
		      		self.tConfig.Position.Right,
		      		self.tConfig.Position.Bottom)
				end
			end
		end
		
	    self.wndMain:Show(false, true)
		
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("redalert", "OnRedAlertOn", self)
		Apollo.RegisterSlashCommand("testu", "OnTestuOn", self)
		Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)
		Apollo.RegisterEventHandler("UnitDestroyed", "OnUnitDestroyed", self)
		Apollo.RegisterSlashCommand("pt", "PrintTable", self)
		Apollo.RegisterSlashCommand("ramove", "OnRedAlertOn", self)
		
		--Timers
		Apollo.CreateTimer("CastCheckTimer", 0.3, true, self)
		Apollo.RegisterTimerHandler("CastCheckTimer", "OnCastCheck", self)

		-- Do additional Addon initialization here
		tUnits = self.tUnits
		
	end
end

-----------------------------------------------------------------------------------------------
-- RedAlert Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/redalert"
function RedAlert:OnRedAlertOn()
	--self.wndMain:Invoke() -- show the window
		if self.wndMain:IsShown() == true then
		Print("RA Hidden")
		self:OnHide()
	else
		Print("RA Shown")
		self:OnShow()
	end
end

function RedAlert:OnTestuOn()
	if tUnits then 
		for i,unit in pairs(tUnits) do
			local strName = unit:GetName()
			if unit:IsCasting() then
				local cast = unit:GetCastName()
				Print(strName.." Casting: "..cast)
			end
			if unit:IsCasting() ~= true then
				Print(strName.." Not Casting")
			end
		end
	end
end

function RedAlert:OnCastCheck()
	local wndAlert = self.wndMain:FindChild("FormLabel")
	if self.tUnits then
		local i = 0
		for idx,unit in pairs(self.tUnits) do
			if unit:IsCasting() then
				local name = unit:GetName()
				local castname = unit:GetCastName()
				--Print(name .. "  " .. castname)
				if self:TableContains(self.tCasts, castname) then
					--Print(name .. " is doing: ".. castname)
					if self.wndMain:IsShown() ~= true then
						self.wndMain:Show(true, true)
					end
					wndAlert:SetText(name .. " > ".. castname)
					i = i + 1
				end
			end
		end
		if i == 0 then 
			self.wndMain:Show(false, true)
			--Print("Should Hide")
		end
	end
end

function RedAlert:OnUnitCreated(unit)
	--Print("Created "..unit:GetName())
	local match = nil
	if self.tUnits then 
		if unit:GetType() == "Player" then
			--Print("Player Created: "..unit:GetName())
			if self:TableNotContains(self.tUnits, unit) then
				--Print("Adding Player to Table: "..unit:GetName())
				table.insert(self.tUnits, unit)
			end
		end
	end
end

function RedAlert:OnUnitDestroyed(unit)
	local match = self:TableContains(self.tUnits, unit)
	local unitType = unit:GetType()
	local unitName = unit:GetName()
	if self.tUnits then 
		if unitType == "Player" then
			if match ~= nil then
				--Print("Removed " .. unitName .." index: " .. match)
				table.remove(self.tUnits, match)
			end
		end
	end
end

function RedAlert:PrintTable()
	for idx,unit in pairs(self.tUnits) do
		Print(unit:GetName())
	end
end

function RedAlert:TableNotContains(table, element)
	for idx,value in pairs(table) do
		if value == element then
			return false
		end
	end
	return true
end

function RedAlert:TableContains(table, element)
	for idx,value in pairs(table) do
		if value == element then
			--Print(idx)
			return idx
		end
	end
	return 
end

function RedAlert:TableContainsBool(table, element)
	for idx,value in pairs(table) do
		if value == element then
			--Print(idx)
			return true
		end
	end
	return 
end

function RedAlert:GetPosition(form)
 	local result = nil
 	if form ~= nil then
  		local leftAnchor, topAnchor, rightAnchor, bottomAnchor = form:GetAnchorOffsets()
  		result =
  		{
   		Bottom = bottomAnchor,
   		Left = leftAnchor,
   		Right = rightAnchor,
   		Top = topAnchor,
  		}
 	end
 	return result
end

function RedAlert:OnWindowMove( wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom )
 	self.tConfig.Position = self:GetPosition(self.wndMain)
end

function RedAlert:OnSave(eType)
	local tSave = {}
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then	
		tSave = self.tConfig
	end
	return tSave
end

function RedAlert:OnRestore(type, savedData)
	if type == GameLib.CodeEnumAddonSaveLevel.Character then
		if savedData ~= nil then
		Print("loading saved data")
		--savedData.Version ~= nil and 
		--savedData.Version == 1 then
			self.tConfig = savedData
		end
	end
end

function RedAlert:OnShow()
	Apollo.StopTimer("CastCheckTimer")
	self.tConfig.IsVisible = true
	self.wndMain:Show(true, false)
end

function RedAlert:OnHide()
	Apollo.StartTimer("CastCheckTimer")
	self.tConfig.IsVisible = false
	self.wndMain:Show(false, false)
end
function RedAlert:OnEsc()
	Apollo.StartTimer("CastCheckTimer")
end

-----------------------------------------------------------------------------------------------
-- RedAlertForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function RedAlert:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function RedAlert:OnCancel()
	self.wndMain:Close() -- hide the window
end


-----------------------------------------------------------------------------------------------
-- RedAlert Instance
-----------------------------------------------------------------------------------------------
local RedAlertInst = RedAlert:new()
RedAlertInst:Init()
